// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################

// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme est une démonstration décrite dans la
// page http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/ScreenShoot.html

// Juste pour faire beau, des animations graphiques

#include "PecheuxGraph.h"
#include <SD.h>

boolean SDabsente=false;
long debut; // nombre de millisecondes depuis le début de la fonction
const long temps=20000; // Durée de chaque animation


//###########################################################################
//####                             Couleurs                              ####
//###########################################################################
// Les appels successifs à gris() retournent d'abord un WHITE, puis des gris de plus en plus clairs jusqu'au BLACK,
// reviennent au WHITE et ainsi de suite
word old_gris=MAX_GREY;
boolean sens_gris=true;
word Gris(void)
{
  if (sens_gris)
    if (old_gris<MAX_GREY) old_gris++;
    else sens_gris=false;
  else
    if (old_gris>0) old_gris--;
    else sens_gris=true;  
  return RGBcolor(old_gris);   
}


// Les appels successifs à color() retournent d'abord un bleu, puis progressivement va au GREEN, puis au RED...
// ainsi de suite
byte couleur_bleue=MAX_GREY, couleur_rouge=0, couleur_verte=0;
char sens_couleur=0;
word color(void)
{
  if (sens_couleur==0) // Va du bleu au cyan
    if (couleur_verte<MAX_GREY) couleur_verte++;
    else sens_couleur++;
  else if (sens_couleur==1) // Va du cyan au vert
    if (couleur_bleue>0) couleur_bleue--;
    else sens_couleur++;
  else if (sens_couleur==2) // Va du vert au jaune
    if (couleur_rouge<MAX_GREY) couleur_rouge++;
    else sens_couleur++;
  else if (sens_couleur==3) // Va du jaune au rouge 
    if (couleur_verte>0) couleur_verte--;
    else sens_couleur++;
  else if (sens_couleur==4) // Va du rouge au magenta
    if (couleur_bleue<MAX_GREY) couleur_bleue++;
    else sens_couleur++;
  else if (sens_couleur==5) // Va du magenta au bleu
    if (couleur_rouge>0) couleur_rouge--;
    else sens_couleur=0;
  return RGBcolor(couleur_rouge,couleur_verte<<1,couleur_bleue);   
}

//###########################################################################
//####                              Babale                               ####
//###########################################################################
void baballe(void)
{
  const char taille=4; // rayon de la balle
  int X,Y, // 16 fois les coordonnés de la balle
      x, y; // Coordonnés de la balle
  int sens_X=16, sens_Y=16; // Sens de déplacement de la balle, entre 4 et 16 ou entre -16 et -4
  const int XMin=taille<<4, // Valeurs des bords de l'écran (fois 16) moins la taille de la balle our qu'elle soit visible
            XMax=(LARGEUR-taille)<<4,
            YMin=taille<<4,
            YMax=(HAUTEUR-taille)<<4;

  clrscr();          
  debut=millis();
  while (debut+temps>millis())
  {
    // Dessine la balle
    x=X>>4; y=Y>>4;
    for (char r=0; r<=taille; r++) circle(x,y,r,GREEN); // En vert
    circle(x,y,taille+1,BLACK); // Ombres noire pour effacer l'ancienne position
    circle(x,y,taille+2,BLACK);
    circle(x,y,taille+3,BLACK);
  
    // Incrémente les déplacements.
    X+=sens_X; // sens_X est entre 4 et 16, X augmente ou diminue de 4 à 16, et l'abscisse x de 0,25 à 1 
    Y+=sens_Y; // Pareil pour y; La balle rebondit un peu au hasard
  
    // Test des bords, on rebondit si on touche un bord
    if (x<taille) sens_X=random(12)+4; // Si on va trop à gauche (x trop petit) , on se déplacera vers la droite (sens_X est positif)
    if (x>LARGEUR-taille) sens_X=-random(12)-4;  // Trop à droite, on ira vers la gauche
    if (y<taille) sens_Y=random(12)+4; // Trop en haut
    if (y>HAUTEUR-taille) sens_Y=-random(12)-4; // Trop en bas
  }
  delay(1000);
}


//###########################################################################
//####                                Star                               ####
//###########################################################################
void uneStar(int x, int y, int couleur) // Dessine une étoile
{
  line(10+x,3+y,24+x,-8+y,couleur); lineTo(6+x,-8+y);
  lineTo(0+x,-25+y);        lineTo(-6+x,-8+y);
  lineTo(-24+x,-8+y);       lineTo(-10+x,3+y);
  lineTo(-15+x,20+y);       lineTo(0+x,10+y);
  lineTo(15+x,20+y);        lineTo(10+x,3+y);
}

void star(void) 
{
  int x,y,couleur; // Paramètres d'une étoile
  setFillMode(STAR); // L'autre mode ne fonctionnerait pas si le fond n'est plus noir

  clrscr();
  debut=millis();
  while (debut+temps>millis())
  {
    // On place une étoile pleine sur l'écran
    uneStar(x=random(LARGEUR-20)+10, y=random(HAUTEUR-20)+10, RANDOM_COLOR); // Dessin de l'étoile creuse
    fill(x, y, getDrawColor()); // Remplissage de l'étoile
  }  
  delay(1000);
}


//###########################################################################
//####                          Tâches de Martin                         ####
//###########################################################################
void martin(void) 
{
  #define TAILLE 4
  byte constante_de_Martin=30;
  int x,y;
  word couleur;
  
  // fond aléatoire, dans les bleus. Cela accélère l'apparition de tâches
  for (int x=0; x<320/TAILLE; x++) for (int y=0; y<320/TAILLE; y++) fillRect(x*TAILLE,y*TAILLE,x*TAILLE+TAILLE,y*TAILLE+TAILLE,random(2)>0?DARK_BLUE:GREEN);
  debut=millis();
  while (debut+temps>millis())
  {
    // Pour aller plus vite, et que c'est plus joli, je ne calcule qu'un quart d'écran et par symétrie, je dessine partout
    for (int x=0; x<DEMI_LARGEUR; x++) for (int y=0; y<DEMI_HAUTEUR; y++) // Pour chaque point du quart haut, gauche
    {
      // Formule de Martin
      couleur=((getPoint((x+DEMI_LARGEUR-1)%DEMI_LARGEUR,y)+getPoint((x+1)%DEMI_LARGEUR,y)+getPoint(x,(y+DEMI_HAUTEUR-1)%DEMI_HAUTEUR)+getPoint(x,(y+1)%DEMI_HAUTEUR))>>2)+constante_de_Martin;
      point(x,y,couleur); // tracé du quart de l'écran 
      point(MAX_X-x,y,couleur); point(MAX_X-x,MAX_Y-y,couleur); point(x,MAX_Y-y,couleur); // Symétries
    }
  }
  delay(1000);
}


//###########################################################################
//####                              Decolore                             ####
//###########################################################################
void decolore(void) 
{
  int x, y; // Coordonnés d'un point quelconque
  // Dessin d'un fond d'écran:
  clrscr();
  fillRect(DEMI_LARGEUR-40, DEMI_HAUTEUR-40, DEMI_LARGEUR, DEMI_HAUTEUR, RED); // En haut à gauche
  fillRect(DEMI_LARGEUR, DEMI_HAUTEUR-40, DEMI_LARGEUR+40, DEMI_HAUTEUR, GREEN); // En haut à droite
  fillRect(DEMI_LARGEUR-40, DEMI_HAUTEUR, DEMI_LARGEUR, DEMI_HAUTEUR+40, BLUE); // En bas à gauche
  fillRect(DEMI_LARGEUR, DEMI_HAUTEUR, DEMI_LARGEUR+40, DEMI_HAUTEUR+40, YELLOW); // En bas à droite

  debut=millis();
  while (debut+temps>millis())
  {
    // Prendre un point au hasard et le redessiner dans la même couleur
    x=random(40)*2; y=random(40)*2;
    // 4 points adjacents
    fillRect(DEMI_LARGEUR+x-40,DEMI_HAUTEUR+y-38,DEMI_LARGEUR+x-39,DEMI_HAUTEUR+y-37,getPoint(DEMI_LARGEUR+x-40,DEMI_HAUTEUR+y-40)); // point au dessus, de quelle couleur? de celle du dessus...
    fillRect(DEMI_LARGEUR+x-40,DEMI_HAUTEUR+y-42,DEMI_LARGEUR+x-39,DEMI_HAUTEUR+y-41); // point en dessous
    fillRect(DEMI_LARGEUR+x-38,DEMI_HAUTEUR+y-40,DEMI_LARGEUR+x-37,DEMI_HAUTEUR+y-39); // point à gauche
    fillRect(DEMI_LARGEUR+x-42,DEMI_HAUTEUR+y-40,DEMI_LARGEUR+x-41,DEMI_HAUTEUR+y-39); // point à droite
  }
  delay(1000);
}


//###########################################################################
//####                              Escargot                             ####
//###########################################################################
void escargot(void)
{
  boolean SDabsente=false; // Si pas le bon fichier, on mettra des caractères au hasard
  File fichier; // Le bon fichier texte contenant les caractères à afficher
  int x1=0, y1=0, x2=LARGEUR, y2=HAUTEUR; // Boîte rétrécisante dans laquelle on peut écrire 
  byte sens=0; // Sens de l'écriture
  String voyelles="aeiou"; // Permet d'avoir plus de voyelles
  String caractere; // Caractère lu pour affichage

  setGraphMode(PAYSAGE); // Initialisation de l'écran
  SDabsente=!initSD(); // Si déja pas de carte...
  if (!SD.exists("DEMOS/TEXTE.TXT")) SDabsente=true; // Le bon fichier n'est pas présent
   else fichier=SD.open("DEMOS/TEXTE.TXT");
  setTextSize(1); // En plus petit, on affiche plus de caractères

  debut=millis();
  while (debut+temps>millis())
  {
    if (SDabsente)
    { // Pas de fichier texte, on affiche des lettres au hasard
      if (!random(5)) text(" "); // 1 chance sur 5 d'avoir un espace (permet d'ailleurs les mots de 10 lettres)
      else if (random(2)>0) text(String(char(random(27)+'a'))); // 1 chance sur 2 d'avoir une lettre quelconque
      else text(String(voyelles[random(5)])); // 1 chance sur 2 d'avoir obligatoirement une voyelle
    }
    else
  { // Le bon fichier texte est présent
    if (!fichier.available()) fichier.seek(0); // Rembobine si on est à la fin
    caractere=String(char(fichier.read())); // On lit un caractère
    if ((byte(caractere[0])==226) || (byte(caractere[0])==194) || (byte(caractere[0])==195)) caractere+=String(char(fichier.read())); // caractère tenant sur 2 octets
    if (byte(caractere[0])==226) caractere+=String(char(fichier.read())); // caractère tenant sur 3 octets: '€'
    if (byte(caractere[0]) == 10) caractere = " ";
    if (byte(caractere[0]) == 13) caractere = "";
    text(caractere); // Lit un caractère sur la SD et l'affiche
    delay(5); // Ralentit un peu
  }
    delay(2); // Ralentit un peu
    switch (sens)
    {
      case EST: if (getTextCursorX()>x2-9) // Si on va trop à droite
        { 
          setTextOrient(++sens); // On écrira vers le bas
          y1+=8; // On n'utilisera plus la première ligne
          setTextCursor(y1,LARGEUR-x2);} // Repositionne le curseur, setTextOrient() le remet à l'origine
        break;
      case SUD: if (getTextCursorX()>y2-9) // Si on va trop ves le bas
        { setTextOrient(++sens); x2-=8; setTextCursor(LARGEUR-x2,HAUTEUR-y2);}  
        break;
      case OUEST: if (getTextCursorX()>LARGEUR-x1-9) // Si on va trop à gauche
        { setTextOrient(++sens); y2-=8; setTextCursor(HAUTEUR-y2,x1);}  
        break;
      case NORD: if (getTextCursorX()>HAUTEUR-y1-9) // Si on va trop en haut
        { setTextOrient(NORMAL); x1+=8; setTextCursor(x1,y1); sens=NORMAL;}  
        break;
    }
    if ((y1>y2-9)||(y1>y2-9)) // Si on n'a plus assez de place dans la boîte 
    {
      x1=0; y1=0; x2=LARGEUR; y2=HAUTEUR; // On reprend la boîte de départ
      setTextOrient(NORMAL); sens=0; //  Sens d'écriture normal
      clrscr(); // Prêt à recommencer
    }  
  }
}


//###########################################################################
//####                              fillRect                             ####
//###########################################################################
void boites(void)
{
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    int x=random(LARGEUR-20); // Coin n'importe ou
    int y=random(HAUTEUR-20);
    fillRect(x,y,x+10+random(10),y+10+random(10),RANDOM_COLOR); // Pavé de 10x10 à 20x20
  }
  delay(1000);
}


//###########################################################################
//####                              Cercles                              ####
//###########################################################################
void cercles(void)
{
  word gris;
  
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    for(int rayon=20; rayon>0; rayon--) // Dessin de cercles concnetriques de plus en plus petits
    {
     gris=Gris(); // Choix de la couleur des cercles que l'on va dessiner
     for (int x=40; x<LARGEUR-20; x+=40) // Pour ce qui tient dans la largeur
       for (int y=40; y<HAUTEUR-20; y+=40) // Pour ce qui tient dans la hauteur
          circle(x,y,rayon,gris); // Dessine tous les cercles
    }  
  }  
  delay(1000);
}


//###########################################################################
//####                            Horizontales                           ####
//###########################################################################
void horizontales(void)
{
  // Ici c'est une figure de lissajous calculée avec des réels
  // Principe: boucle est en fait un angle. On dessine "boucle", et on efface "boucle-200". Il reste donc 200 lignes affichées
  int abscisse_affiche, ordonnee_affiche, abscisse_efface, ordonnee_efface;
  int boucle=1000;
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    // Calcul des coordonnés de lissajous
    abscisse_affiche=int(sin(float(++boucle)/181.0f)*float((LARGEUR-20)/2)*0.99)+((LARGEUR-20)/2);
    abscisse_efface=int(sin(float(boucle-200)/181.0f)*float((LARGEUR-20)/2)*0.99)+((LARGEUR-20)/2);
    ordonnee_affiche=int(sin(float(boucle)/61.0f)*float(HAUTEUR/2)*0.99)+(HAUTEUR/2);
    ordonnee_efface=int(sin(float(boucle-200)/61.0f)*float(HAUTEUR/2)*0.99)+(HAUTEUR/2);
    // Effacement d'une vieille ligne
    hLine(abscisse_efface,abscisse_efface+19,ordonnee_efface,BLACK);
    // Dessin d'une nouvelle
    hLine(abscisse_affiche,abscisse_affiche+19,ordonnee_affiche,color());
  }  
  delay(1000);
}


//###########################################################################
//####                               Lignes                              ####
//###########################################################################
void lignes(void)
{
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    for(int boucle=0; boucle<=MAX_X; boucle++) line(0,0,boucle,MAX_Y,Gris()); // Premier évantail
    for(int boucle=MAX_Y; boucle>=0; boucle--) line(0,0,MAX_X,boucle,Gris()); // Deuxième évantail...
    for(int boucle=0; boucle<MAX_Y; boucle++) line(MAX_X,0,0,boucle,Gris());
    for(int boucle=0; boucle<=MAX_X; boucle++) line(MAX_X,0,boucle,MAX_Y,Gris());
    for(int boucle=MAX_X; boucle>=0; boucle--) line(MAX_X,MAX_Y,boucle,0,Gris());
    for(int boucle=0; boucle<MAX_Y; boucle++) line(MAX_X,MAX_Y,0,boucle,Gris());
    for(int boucle=MAX_Y; boucle>=0; boucle--) line(0,MAX_Y,MAX_X,boucle,Gris()); 
    for(int boucle=MAX_X; boucle>=0; boucle--) line(0,MAX_Y,boucle,0,Gris());
  }  
  delay(1000);
}


//###########################################################################
//####                             Rectangles                            ####
//###########################################################################
void rectangles(void)
{
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    for(char boucle=0; boucle<100; boucle++) // On dessine 100 rectangles
    {
      int x=random(LARGEUR-20); // N'importe ou
      int y=random(HAUTEUR-20);
      rect(x,y,x+10+random(10),y+10+random(10),RANDOM_COLOR); // de taille 10x10 à 20x20
    }
    delay(500);
    clrscr(); 
  }  
  delay(1000);
}


//###########################################################################
//####                                Arcs                               ####
//###########################################################################
void arcs(void)
{
  // On va remplacer les arcs les uns après les autres: on trace un bout de noir (pour effacer) et un bout de blanc
  // Cela va suffisament vite pour que l'on croit à une rotation
  word courbe=0;
  clrscr(); 
  debut=millis();
  while (debut+temps>millis())
  {
    for (int x=60; x<LARGEUR-20; x+=60) for (int y=60; y<HAUTEUR-20; y+=60) for (int rayon=3; rayon<40; rayon+=4)
    {
      arc(x,y,rayon,(((courbe-1)&0xf)<<4)+(courbe&0xf),BLACK); // Efface un ancien arc
      arc(x,y,rayon,((courbe&0xf)<<4)+((courbe+1)&0xf),WHITE); // Dessine un nouvel arc
    }
    courbe++;
  }  
  delay(1000);
}


//###########################################################################
//####                              Images                               ####
//###########################################################################
void images(void)
{
  if (SDabsente) return; // si pas de carte, c'est pas la peine d'attendre
  debut=millis();
  while (debut+temps>millis())
  {
    // 2 grandes images prises au hasard sur la carte, on affiche qu'une seule par boucle
    bpxDraw("320x240/"+String(random(45))+".bpx",0,0); // Affiché seulement en mode paysage, sinon, elle ne passe pas
    bpxDraw("240x320/"+String(random(37))+".bpx",0,0); // Affiché seulement en mode portrait (on en affiche donc une des deux!)
  }    
  delay(1000);
}


//###########################################################################
//####                                 XP                                ####
//###########################################################################
void xp(void)
{
  byte x=99, sens=-1;
  
  if (SDabsente) return; // si pas de carte, c'est pas la peine d'attendre
  clrscr();
  bpxDraw("DEMOS/XP.BPX",56,20);
//  bpxSave("DEMOS/XP.BPX",56,20,263,166); // Affiché seulement en mode paysage, sinon, elle ne passe pas
  rect(97,212,210,220,GREY); //266 218
  debut=millis();
  while (debut+temps>millis())
  {
    fillRect(x,213,x+16,219, BLACK); // Effacer l'ancien
    x+=sens;
    if ((x==98)||(x==193)) sens=-sens;
    fillRect(x,213,x+4,219,BLUE);
    fillRect(x+6,213,x+10,219,BLUE);
    fillRect(x+12,213,x+16,219,BLUE);
    delay(10);
  }    
  delay(1000);
}


//###########################################################################
//####                             Spirograph                            ####
//###########################################################################
void spirographe(void)
{
  float increment=PI/3000; // Increment de l'angle
  float angle=0; // Qui va croître
  int vitesse1, // Le point A tourne d'un angle angle*vitesse1
      rayon1, // Et à une distance de rayon1 du centre de l'écran
      vitesse2, // Le point B tourne d'un angle angle*vitesse2
      rayon2; // Et à une distance de rayon2 du point A
  long debut_spirographe;

  debut=millis();
  while (debut+temps>millis())
  {
    rayon1=random(70)+50;  // 50 à 119 
    rayon2=119-rayon1; // le compément à 119 pour avoir le maximum d'espace occupé
    vitesse1=random(6)+1; // entre 1 et 6
    vitesse2=random(6)+10; // entre 10 et 16
    clrscr();
    debut_spirographe=millis();
    while (debut_spirographe+5000>millis())
    {
      point(DEMI_LARGEUR+trunc(rayon1*cos(angle*vitesse1)+rayon2*cos(angle*vitesse2)), DEMI_HAUTEUR+trunc(rayon1*sin(angle*vitesse1)+rayon2*sin(angle*vitesse2)));                                       
      angle+=increment;
    }
  }  
}



//###########################################################################
//####                           Setup et loop                           ####
//###########################################################################

void setup() 
{
  setGraphMode(PAYSAGE); // Initialisation générale
  if (!initSD())
  {
    text(F("Avec  une carte  SD  contenant lecontenu   de   répertoire   SD   ladémonstration est  plus  complète.\nVous n'aurez pas tout."));
    delay(5000); 
    SDabsente=true;
  }
  else
  {    if (!SD.exists("320x240/1.BPX"))
    {
      text(F("Je  ne  trouve  pas  sur  la  cartemémoire le  contenu   de   SD.  Ladémonstration  ne  sera  donc pascomplète."));
      delay(5000);
      SDabsente=true;
    }
  }
}

void loop()
{
  spirographe();
  baballe();
  star();
  martin();
  decolore();
  escargot();
  boites();
  cercles();
  horizontales();
  lignes();
  xp();
  rectangles();
  arcs();
  images();
}
