// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################


// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la classe
// Timer décrites dans la page  
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/TimerExemple.html


// Ce programme dessine un sapin de Noël puis 35 métronomes vont faire
// changer de couleur les 35 boules de Noël à des instants légèrement
// différents (entre 2 et 2,1 seconde). Au début, les boules changent de
// couleur presque en même temps, puis à la longue, elles vont devenir
// indépendantes
// Avec un Mega, on peut mettre 250 métronomes

#include <PecheuxGraph.h>

// Comme le comportement de chaque métronome est légèrement différent, a
// savoir que la boule associée est positionnée différemment pour chacun
// d'eux, il faut que chaque métronome mémorise ces coordonnes. Dériver
// une nouvelle classe est une bonne solution
const word couleurs[]={RED, GREEN, BLUE, CYAN, MAGENTA, YELLOW};
class Boule: public Clock
{
  public:
    Boule(int x, int y, unsigned long duree): Clock(duree)
    {
      this->x = x; this->y = y; // Dans ce constructeur, on rajoue l'initialisation des coordonnées
    }
    int x; int y; // Coordonnée de la boule associée
    virtual void onTime(void) // Nouveau comportement
    {
      fillRect(x, y, x+2, y+2, couleurs[random(6)]); //On change de couleur la boule associée (5 chances sur 6!)
    }
};


void setup()
{
  long x, y; // Coordonnés d'un point
  setGraphMode(PORTRAIT);

  // Le sapin
  if (bpxDraw("DEMOS/NOEL.BPX", 0, 0)) // Photo d'un sapin
  { // Pas de carte ou pas de fichier, on va tout dessiner à la main
    clrscr(LIGHT_YELLOW); // Mur
    line(0, 240, 134, 223, BISQUE); lineTo(239,258); fill(134,250, BISQUE); // Sol
    line(121, 38, 50, 241, DARK_GREEN); lineTo(177, 241); lineTo(121, 38); // Contour du sapin
    fill(121, 50, DARK_GREEN); fill(121, 239, DARK_GREEN); //Vert du sapin
    fillRect(110,240,132,290,DARK_RED); // Tronc
  }

  // Déclaration des 40 métronomes. Comme il y en a beaucoup, une boucle
  // est bienvenue. On est donc obligé d'avoir une déclaration dynamique. On
  // n'a pas besoin du pointeur créé, on n'en utilise pas
  for (byte nombre = 0; nombre < 35; nombre++) // Au dela de 40 avec un Uno, la pile déborde et le programme plante
  {
    do
    {
      x = random(178); // Coordonnées au hasard, mais pour l'instant cela risque de dépasser du sapin
      y = random(242); 
    }
    // Pour la condition pas trop à gauche, le sapin étant vaguement
    // triangulaire, il ne faut pas que les points soient à gauche de la
    // droite passant par (121,38) et (50,241). Il faut calculer l'équation
    // de la droite du type ax+by+c; si c'est nul, on est sur la droite, si
    // c'est négatif, on est dans un des deux demi-plan, si c'est positif on
    // est dans l'autre demi-plan. Un essai nous dira si on a pris le bon.
    // le point (x,y) est sur la droite passant par (x1,y1) et (x2,y2) si
    //   x - x1      x1 - x2
    //   ------  =   -------
    //   y - y1      y1 - y2
    // L'équation de la droite recherchée est:
    // (x - x1)*(y1 - y2) - (y - y1)*(x1 - x2) = 0
    // on testera si le membre de droite est nul (on est sur la limite du sapin)
    //                                       positif (on est d'un côté)
    //                                       négatif (on est de l'autre côté)
    while ((203*x+71*y-27261<=0) // Pas trop à gauche
           || (203*x-56*y-22425>=0)); // Pas trop à droite
    new Boule(x-1, y-1, 500 + nombre); // (x,y) étant dans le sapin, on créé le métronome
  }
}

void loop()
{
  scanEvent(); // Gestion des boutons et des horloges, le plus souvent seul dans loop()
}
