// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################


// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la classe
// PushZone décrites dans la page
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/PushZoneExemple.html

// Mise em place de 3 bouton de type PushZone (pousoir sur une zone, pas de
// dessin dans le bouton). Quand on appuie sur un bouton, un disque blanc
// montre son activité

// Version 2:
// - Les boutons ont une définition dynamique avant le setup()
// - Utilise onSelectFunction pour exécuter l'action à faire

#include <PecheuxGraph.h> // Appel de la bibliothèque 


// Déclaration dynamique de l'instance, peut se faire en dehors du setup
// Dans ce cas bouton est accessible partout
PushZone boutonHaut(200, 40, 240, 80); // Bouton en haut
PushZone boutonCentre(200, 100, 240, 140); // Bouton au centre
PushZone boutonBas(200, 160, 240, 200); // Bouton en bas


// Déclaration d'une fonction qui fera ce qu'il faut faire à chaque changement
// Cette fonction ne doit pas avoir de paramètres et ne doit rien retourner
// Le nom peut être quelconque
// Je passe par les pointeurs exceptionnellement pour s'affranchir des noms des
// boutons. Cela permet aussi de montrer l'utilité du pointeru premeirControle.
// Si cela vous pose problème, laissez de côté la compréhension de cette fonction
void Action() // Action à faire quand on appuie sur un bouton
{
  PushZone *pointeur = premierControle;
  int ordonnee = 180; // Ligne ou se trouve le contrôle. On commence par celui du bas, c'est dle dernier défini
  while (pointeur != NULL) // tant qu'on a pas fait toute la liste
  {
    if (pointeur->isSelected()) fillCircle(100, ordonnee, 20, WHITE); // Disque blanc si il est sélectionné
    else fillCircle(100, ordonnee, 20, BLACK);

    pointeur = pointeur->controleSuivant; // mise à jour pour le bouton suivant
    ordonnee -= 60;
  }
}


void setup()
{
  setGraphMode(PAYSAGE); // Initialisation de la carte
  rect(200, 40, 240, 80, GREEN); // Carré du haut
  rect(200, 100, 240, 140, GREEN); // Carré au centre de l'écran
  rect(200, 160, 240, 200, GREEN); // Carré du bas
  text(F("Appuyez sur un carré"));

  // Association de la fonction qui va faire le travail avec le bouton
  // Attention, il y a un caractère "&" avant le nom de la fonction.
  // Cette ligne veut dire: bouton a une propriété onSelectFunction et on va
  // affecter cette propriété avec l'adresse (présence du "&") de onSelectAction
  boutonHaut.onSelectFunction = &Action; // Fonction appelée si on active le bouton du haut de l'écran
  boutonCentre.onSelectFunction = &Action; // Fonction appelée si on active le bouton au milieu de l'écran
  boutonBas.onSelectFunction = &Action; // Fonction appelée si on active le bouton en bas de l'écran
  boutonHaut.onUnselectFunction = &Action; // Fonction appelée si on désactive le bouton du haut de l'écran
  boutonCentre.onUnselectFunction = &Action; // Fonction appelée si on désactive le bouton au milieu de l'écran
  boutonBas.onUnselectFunction = &Action; // Fonction appelée si on désactive le bouton en bas de l'écran
}


void loop()
{
  scanEvent(); // Gestion des boutons et des horloges, le plus souvent seul dans le loop
}
