// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################


// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la classe
// CheckZone décrites dans la page
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/CheckZoneExemple.html

// Mise em place de 3 bouton de type CheckZone (zone à cocher, pas de dessin
// dans le bouton). Quand on appuie sur un bouton, un disque blanc montre son
// activité

// Version 3:
// - Les boutons ont une définition statique
// - Utilise onSelectFunction pour exécuter l'action à faire
// - Une seule action pour les trois boutons, différence faites sur les coordonnées

#include <PecheuxGraph.h> // Appel de la bibliothèque 


// Déclaration statique des instances, doit se faire en dehors du setup
// Dans ce cas les boutons sont accessibles partout
CheckZone boutonHaut(200,40,240,80); // Bouton en haut
CheckZone boutonCentre(200,100,240,140); // Bouton au centre
CheckZone boutonBas(200,160,240,200); // Bouton en bas


// Déclaration d'une fonction qui fera ce qu'il faut faire à chaque changement
// Cette fonction ne doit pas avoir de paramètres et ne doit rien retourner
// Le nom peut être quelconque
void onSelectAction() // Action à faire quand on appuie sur un bouton
{
  int y=getTouchY();
  if (y<90) // On a touché sur le bouton du haut
  {
    if (boutonHaut.isSelected()) fillCircle(100,60,20,WHITE); // On s'occupe du premier bouton
    else fillCircle(100,60,20,BLACK);
  }
  else if (y>150) // On a touché sur le bouton du bas
  {
    if (boutonBas.isSelected()) fillCircle(100,180,20,WHITE); // Puid du troisième
    else fillCircle(100,180,20,BLACK);
  }
  else // on a donc forcément touché sur le bouton du milieu
  { // (Accolades inutiles pour le dernier cas)
    if (boutonCentre.isSelected()) fillCircle(100,120,20,WHITE); // Puis du deuxème
    else fillCircle(100,120,20,BLACK);
  }  
}


void setup()
{
  setGraphMode(PAYSAGE); // Initialisation de la carte
  rect(200,40,240,80,GREEN); // Carré du haut
  rect(200,100,240,140,GREEN); // Carré au centre de l'écran
  rect(200,160,240,200,GREEN); // Carré du bas
  text(F("Appuyez sur un carré"));
  
  boutonHaut.onSelectFunction=  // Fonction appelée si on active le bouton du haut de l'écran
  boutonCentre.onSelectFunction= // Fonction appelée si on active le bouton au milieu de l'écran
  boutonBas.onSelectFunction= // Fonction appelée si on active le bouton en bas de l'écran
  boutonHaut.onUnselectFunction= // Fonction appelée si on désactive le bouton du haut de l'écran
  boutonCentre.onUnselectFunction= // Fonction appelée si on désactive le bouton au milieu de l'écran
  boutonBas.onUnselectFunction=&onSelectAction; // Fonction appelée si on désactive le bouton en bas de l'écran
}


void loop() 
{
  scanEvent(); // Gestion des boutons et des horloges, le plus souvent seul dans le loop
}
