// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################


// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la classe
// CheckZone décrites dans la page
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/CheckZoneExemple.html

// Mise em place de 3 bouton de type CheckZone (zone à cocher, pas de dessin
// dans le bouton). Quand on appuie sur un bouton, un disque blanc montre son
// activité

// Version 5:
// - Les boutons ont une définition statique
// - Teste dans le loop sans utiliser onSelectFunction
// - On mémorise les actions faites pour ne pas surcharger loop()

#include <PecheuxGraph.h> // Appel de la bibliothèque 


// Déclaration statique des instances, doit se faire en dehors du setup
// Dans ce cas les boutons sont accessibles partout
CheckZone boutonHaut(200,40,240,80); // Bouton en haut
CheckZone boutonCentre(200,100,240,140); // Bouton au centre
CheckZone boutonBas(200,160,240,200); // Bouton en bas


// Pour mémoriser les actions faites, on a recours à trois variables
// qui vont permettre de savoir si un bouton a changé
boolean ancienEtatBoutonHaut; // ancien état du bouton du haut
boolean ancienEtatBoutonCentre; // ancien état du bouton du centre
boolean ancienEtatBoutonBas; // ancien état du bouton du bas


void setup()
{
  setGraphMode(PAYSAGE); // Initialisation de la carte
  rect(200,40,240,80,GREEN); // Carré du haut
  rect(200,100,240,140,GREEN); // Carré au centre de l'écran
  rect(200,160,240,200,GREEN); // Carré du bas
  text(F("Appuyez sur un carré"));
  }


void loop() 
{
  scanEvent(); // Gestion des boutons et des horloges, d'habitude seul dans le loop

  // On va tester systématiquement l'état des boutons pour savoir si l'un d'eux a changé.
  // Si c'est le cas, on agit.
  // Ici on surcharge peu la boucle loop(), le dessin est fait que si nécessaire.
  if (boutonHaut.isSelected()!=ancienEtatBoutonHaut)
  { // Le bouton haut a changé d'état, on le redessine
    if (boutonHaut.isSelected()) fillCircle(100,60,20,WHITE); // On s'occupe du premier bouton
    else fillCircle(100,60,20,BLACK);
    ancienEtatBoutonHaut=boutonHaut.isSelected(); // Et on met à jour la mémoire
  }
  
  if (boutonCentre.isSelected()!=ancienEtatBoutonCentre)
  { // Le bouton du centre a changé d'état, on le redessine
    if (boutonCentre.isSelected()) fillCircle(100,120,20,WHITE); // Puid du troisième
    else fillCircle(100,120,20,BLACK);
    ancienEtatBoutonCentre=boutonCentre.isSelected(); // Et on met à jour la mémoire
  }
  
  if (boutonBas.isSelected()!=ancienEtatBoutonBas)
  { // Le bouton bas a changé d'état, on le redessine
    if (boutonBas.isSelected()) fillCircle(100,180,20,WHITE); // Puis du deuxème
    else fillCircle(100,180,20,BLACK);
    ancienEtatBoutonBas=boutonBas.isSelected(); // Et on met à jour la mémoire
  }
}
