// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################

// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la classe
// RadioZone décrites dans la page
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/RadioExemple.html

// L'intérieur d'une maison est dessinée. Trois boutons de type Radio
// permettent d'ouvrir ou de fermer les volets roulants

#include <PecheuxGraph.h>

// Equivalece des constantes
const byte MONTE = 0;
const byte STOP = 2;
const byte DESCEND = 1;

// Définition des contrôles, doit se faire avant setup
Clock horloge(100); // Le volet se déplace toutes les 100ms
RadioZone boutonMonte(184, 115, 210, 127, MONTE); // Définit une zone cliquable, notre bouton monte
RadioZone boutonStop(184, 129, 210, 141, STOP); // Définit une zone cliquable, notre bouton stop
RadioZone boutonDescend(184, 143, 210, 155, DESCEND); // Définit une zone cliquable, notre bouton descend

// Variables et constantes pour le volet roulant
const word BLEU_CIEL = 0x755E; // Couleur du ciel
const byte HAUT = 55; // Le haut de la fenêtre
const byte BAS = 155; // le bas de la fenêtre
byte position = HAUT - 1; // Position du volet (plus haut que HAUT, donc complètement rentré)


// Fonction appelée tous les 100 ms pour redessiner les volets
void onTimeVolets(void)
{
  switch (getRadioValeur()) // Gestion du mouvement
  {
    case MONTE:
      if (position >= HAUT) position--; // Monte le volet d'un pixel si il n'est pas en haut
      else // on vient d'arriver en haut
      {
        // Désactivation du bouton haut, choisir une des trois possibilités (une seule sans commentaires)
        // getRadioPointeur()->unselect(); // Désactive le bouton haut
        unselectRadio(); // Désactive tous les boutons
        //static_cast <RadioZone *> (premierControle())->select(); // Active le bouton stop, cela fonctionne car il a été défini en dernier
        fill(160, 30, BLACK); // On éteint la lampe
      }
      break;
    case DESCEND:
      if (position < BAS) position++; // Descend le volet d'un pixel si il n'est pas en haut
      else // on vient d'arriver en bas
      {
        // Désactivation du bouton bas, choisir une des trois possibilités (une seule sans commentaires)
        // getRadioPointeur()->unselect(); // Désactive le bouton bas
        unselectRadio(); // Désactive tous les boutons
        //static_cast <RadioZone *> (premierControle())->select(); // Active le bouton stop, cela fonctionne car il a été défini en dernier
        fill(160, 30, YELLOW); // On allume la lampe
      }
      // case STOP: // On ne fait pas bouger le volet, rien à faire.
  }
  // Dessin du volet
  if ((getRadioValeur() == MONTE) || (getRadioValeur() == DESCEND)) // Sinon cela clignotte aussi à l'arrêt
  {
    // Dessine toutes les lattes en marron
    fillRect(50, HAUT, 107, position, DARK_RED);
    fillRect(114, HAUT, 170, position, DARK_RED);
    // dessine une latte sur 16 en noir
    for (byte latte = 0; latte <= position - HAUT; latte += 16) // N° des lattes à dessiner
    { // Une latte sur 16 est noire
      hLine(50, 107, position - latte, BLACK); // Pour la vitre de gauche
      hLine(114, 170, position - latte); // Pour la vitre de droite
    }
    // Efface la latte en dessous (au cas ou on remonte le volet)
    if (position != BAS) // Seulement si on n'est pas tout en bas, sinon on dessinerait dans le cadre
    {
      hLine(50, 107, position + 1, BLEU_CIEL); // Pour la vitre de gauche
      hLine(114, 170, position + 1); // Pour la vitre de droite
    }
  }
}


void setup()
{
  horloge.onTimeFunction = &onTimeVolets; // Action à faire quand l'hrologe donne son tip

  setGraphMode(PAYSAGE); // Initialisation
  setTextCursor(10, 200); // Consigne
  text(F(" Appuyez sur les boutons,\n      près de la porte"));

  maison(0, 3); // Avec 3 boutons à droite
  fillRect(50, 55, 107, 155, BLEU_CIEL); // Vitre de gauche, fond uni
  fillRect(114, 55, 170, 155); // Vitre de droite, fond uni
}


void loop()
{
  scanEvent(); // Gestion des boutons et des horloges, le plus souvent seul dans le loop
}
