// Version 3.0.0

//###########################################################################
//###########################################################################
//####                                                                   ####
//####                           Olivier Pécheux                         ####
//####                          Olivier@Pecheux.fr                       ####
//####                         (33) +6 69 77 82 58                       ####
//####         http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph        ####
//####                                                                   ####
//###########################################################################
//###########################################################################


// Ceci est un complément aux pages de définition et d'explications de la
// librairie PecheuxGraph. Ce programme illustre l'utilisation de la fonction
// unselectRadio() décrite dans la page
// http://dansetrad.fr/Orgue_de_barbarie/PecheuxGraph/Definition/unselectRadioExemple.html

// Mise em place de 3 boutons de type RadioZone (bouton radio, pas de dessin
// dans le bouton). Quand on appuie sur un bouton, un disque blanc montre son
// activité

// Version 1:
// - Les boutons ont une définition statique
// - Utilise unselectRadio pour tout désactiver

#include <PecheuxGraph.h> // Appel de la bibliothèque 


// Déclaration statique des instances, doit se faire en dehors du setup
// Dans ce cas les boutons sont accessibles partout
RadioZone boutonHaut(200, 40, 240, 80, 1); // Bouton en haut
RadioZone boutonCentre(200, 100, 240, 140, 2); // Bouton au centre
RadioZone boutonBas(200, 160, 240, 200, 3); // Bouton en bas
PushZone boutonEfface(280, 200, 319, 239); // Bouton pour effacer (bouton normal!)


// Déclaration de la fonction qui redessinent les boutons
// Cette fonction ne doivent pas avoir de paramètres et ne doit rien retourner
// Le nom peut être quelconque
void action(void)
{
  fillRect(80, 40, 120, 200, BLACK); // Efface tous les disques
  // Pour dessiner le disque, on le place à la même ordonnée que le bouton
  fillCircle(100, getRadioValeur() * 60, 20, WHITE);
}

// Fonction pour effacer
void efface(void)
{
  unselectRadio(); // Déactive le bouton 
  action(); // Redessine les boutons
}


void setup()
{
  setGraphMode(PAYSAGE); // Initialisation de la carte

  // Fond d'écran
  rect(200, 40, 240, 80, GREEN); // Carré du haut
  rect(200, 100, 240, 140); // Carré au centre de l'écran
  rect(200, 160, 240, 200); // Carré du bas
  text(F("Appuyez sur un carré"));
  rect(280, 200, 319, 239); // Pour effacer
  setTextKeep(CUT);
  setTextCursor(290,205); setTextSize(5); setTextColor(RED); text("X"); // Croix
  
  boutonHaut.onSelectFunction = &action; // Fonction appelée si on active le bouton du haut de l'écran
  boutonCentre.onSelectFunction = &action; // Fonction appelée si on active le bouton au milieu de l'écran
  boutonBas.onSelectFunction = &action; // Fonction appelée si on active le bouton en bas de l'écran
  boutonEfface.onSelectFunction = &efface; // Fonction appelée si on active le bouton d'effacement
}


void loop()
{
  scanEvent(); // Gestion des boutons et des horloges, le plus souvent seul dans le loop
}
